@@ -85,16 +85,14 @@ psd_pseudop = \(y,x,wt,cores = 1,
 #' Xuezhi Cang & Wei Luo (2018) Spatial association detector (SPADE),International
 #' Journal of Geographical Information Science, 32:10, 2055-2075, DOI:  10.1080/13658816.2018.1476693
 #'
-#' @param formula A formula of calculate power of spatial and multilevel discretization determinant \eqn{PSMDQ_s}.
-#' @param data A data.frame or tibble of observation data.
-#' @param wt (optional) The spatial weight matrix.When `wt` is not provided, must provide `locations`.
-#' And `gdverse` will use `locations` columns to construct spatial weight use `inverse_distance_weight()`.
-#' @param locations (optional) The geospatial locations coordinate columns name which in `data`.
-#' Useful and must provided when `wt` is not provided.
-#' @param discnum (optional) Number of multilevel discretization.Default will use `3:22`.
+#' @param yobs Variable Y
+#' @param xobs The original undiscretized covariable X.
+#' @param wt The spatial weight matrix.
+#' @param discnum (optional) Number of multilevel discretization. Default will use `3:22`.
 #' @param discmethod (optional) The discretization methods. Default will use `quantile`.
-#' Noted that `robust` will use `robust_disc()`; `rpart` will use `rpart_disc()`;
-#' Others use `st_unidisc()`.
+#' If `discmethod` is set to `robust`, the function `robust_disc()` will be used. Conversely,
+#' if `discmethod` is set to `rpart`, the `rpart_disc()` function will be used. Others use
+#' `st_unidisc()`. Currently, only one `discmethod` can be used at a time.
 #' @param cores (optional) A positive integer(default is 1). If cores > 1, use parallel computation.
 #' @param seed (optional) Random seed number, default is `123456789`.
 #' @param permutations (optional) The number of permutations for the PSD computation. Default is `0`,
@@ -106,13 +104,13 @@ psd_pseudop = \(y,x,wt,cores = 1,
 #'
 #' @examples
 #' data('sim')
-#' psmd_pseudop(y ~ .,
-#'              data = dplyr::select(sim,1:4),
-#'              locations = c('lo','la'))
+#' wt = inverse_distance_weight(sim$lo,sim$la)
+#' psmd_pseudop(sim$y,sim$xa,wt)
 #'
-psmd_pseudop = \(formula,data,wt = NULL,locations = NULL,discnum = NULL,discmethod = NULL,
-                 cores = 1, seed = 123456789, permutations = 0, ...){
-  qs = psmd_spade(formula,data,wt,locations,discnum,discmethod,cores,seed,...)
+psmd_pseudop = \(yobs, xobs, wt, discnum = 3:22,
+                 discmethod = 'quantile', cores = 1,
+                 seed = 123456789, permutations = 0, ...){
+  qs = psmd_spade(yobs,xobs,wt,discnum,discmethod,cores,seed,...)
   if (permutations == 0){
     fd = tibble::tibble("Q-statistic" = qs, "P-value" = "No Pseudo-P Value")
   } else {
@@ -130,26 +128,19 @@ psmd_pseudop = \(formula,data,wt = NULL,locations = NULL,discnum = NULL,discmeth
       on.exit(parallel::stopCluster(cores), add=TRUE)
     }
 
-    formula = stats::as.formula(formula)
-    formula.vars = all.vars(formula)
-    if (formula.vars[2] == "."){
-      if (length(!(which(colnames(data) %in% c(formula.vars[1],locations)))) > 1) {
-        stop('please only keep `dependent` and `independent` columns in `data`; When `wt` is not provided, please make sure `locations` coordinate columns is also contained in `data` .')
-      } else {
-        xname = colnames(data)[-which(colnames(data) %in% c(formula.vars[1],locations))]
-      }
-    } else {
-      xname = formula.vars[2][-which(formula.vars[2] %in% c(formula.vars[1],locations))]
-    }
-    data = as.data.frame(data)
     set.seed(seed)
     randomnum = stats::runif(1)
-    xperm = shuffle_vector(data[,xname,drop = TRUE],randomnum,seed = seed)
-    yperm = shuffle_vector(data[,formula.vars[1],drop = TRUE],randomnum,seed = seed)
+    xperm = shuffle_vector(xobs,randomnum,seed = seed)
+    yperm = shuffle_vector(yobs,randomnum,seed = seed)
+    wt_perm = wt
+    seedn = seed
+    discn = discnum
+    discm = discmethod
     calcul_psmd = \(p_shuffle){
-      data[,xname] = shuffle_vector(xperm,p_shuffle[[1]],seed = seed)
-      data[,formula.vars[1]] = shuffle_vector(yperm,p_shuffle[[2]],seed = seed)
-      return(psmd_spade(formula,data,wt,locations,discnum,discmethod,cores=1,seed,...))
+      xperm_new = shuffle_vector(xperm,p_shuffle[[1]],seed = seed)
+      yperm_new = shuffle_vector(yperm,p_shuffle[[2]],seed = seed)
+      return(psmd_spade(yperm_new,xperm_new,wt_perm,discn,discm,
+                        cores = 1,seed = seedn, ...))
     }
 
     if (doclust) {
