@@ -2,27 +2,22 @@
 #' @author Wenbo Lv \email{lyu.geosocial@gmail.com}
 #' @description
 #' Function for spatial association detector (SPADE) model.
-#' @note
-#' The columns in the `locations` part of data are only used to construct spatial weight matrix
-#' when `wt` is `NULL`, and are not considered as explanatory variables. If you need to include
-#' spatial locations as explanatory variables, build a spatial weight matrix ahead of time,
-#' leaving the `locations` parameter to `NULL`.The most recommended method is to explicitly
-#' specify all variables in `formula` instead of using `.`!
 #'
 #' @references
 #' Xuezhi Cang & Wei Luo (2018) Spatial association detector (SPADE),International
 #' Journal of Geographical Information Science, 32:10, 2055-2075, DOI:  10.1080/13658816.2018.1476693
 #'
 #' @param formula A formula of spatial association detector (SPADE) model.
-#' @param data A data.frame or tibble of observation data.
-#' @param wt (optional) The spatial weight matrix. When `wt` is not provided, must provide `locations`.
-#' And `gdverse` will use `locations` columns to construct spatial weight use `inverse_distance_weight()`.
-#' @param locations (optional) The spatial location coordinates columns name in `data`.
-#' Useful and must provided when `wt` is not provided. When `wt` is provided, `locations` is not need.
+#' @param data A data.frame, tibble or sf object of observation data.
+#' @param wt (optional) The spatial weight matrix. When `data` is not an `sf` object, must provide `wt`.
+#' @param discvar (optional) Name of continuous variable columns that need to be discretized. Noted that
+#' when `formula` has `discvar`, `data` must have these columns. By default, all independent variables are
+#' used as `discvar`.
 #' @param discnum (optional) Number of multilevel discretization. Default will use `3:22`.
-#' @param discmethod (optional) The discretization methods. Default all use `quantile`.
-#' Noted that `robust` will use `robust_disc()`; `rpart` will use `rpart_disc()`;
-#' Others use `st_unidisc()`.
+#' @param discmethod (optional) The discretization methods. Default all use `quantile`. Note
+#' that when using different `discmethod` for `discvar`, please ensure that the lengths of
+#' both are consistent. Noted that `robust` will use `robust_disc()`; `rpart` will use
+#' `rpart_disc()`; Others use `st_unidisc()`.
 #' @param cores (optional) A positive integer(default is 1). If cores > 1, use parallel computation.
 #' @param seed (optional) Random number seed, default is `123456789`.
 #' @param permutations (optional) The number of permutations for the PSD computation. Default is `0`,
@@ -37,30 +32,58 @@
 #'
 #' @examples
 #' data('sim')
-#' g = spade(y ~ ., data = sim,
-#'           locations = c('lo','la'),
-#'           discvar = c("xa","xb","xc"))
+#' sim1 = sf::st_as_sf(sim,coords = c('lo','la'))
+#' g = spade(y ~ ., data = sim1)
 #' g
 #'
-spade = \(formula, data, wt = NULL,
-          locations = NULL, discnum = NULL, discmethod = NULL,
-          cores = 1, seed = 123456789, permutations = 0, ...){
+spade = \(formula, data, wt = NULL, discvar = NULL, discnum = 3:22,
+          discmethod = 'quantile', cores = 1, seed = 123456789, permutations = 0, ...){
   formula = stats::as.formula(formula)
   formula.vars = all.vars(formula)
+  if (inherits(data,'sf')) {
+    if (is.null(wt)){
+      wt_spade = sdsfun::inverse_distance_swm(data)
+    } else {
+      wt_spade = wt
+    }
+    data = sf::st_drop_geometry(data)
+  } else if (inherits(data,'data.frame')) {
+    if (is.null(wt)){
+      stop("When `data` is `data.frame` or `tibble`, please provide `wt` in spade input!")
+    } else {
+      wt_spade = wt
+    }
+  }
   if (formula.vars[2] != "."){
-    data = dplyr::select(data,dplyr::all_of(c(formula.vars,locations)))
+    data = dplyr::select(data,dplyr::all_of(formula.vars))
   }
   yname = formula.vars[1]
-  xname = colnames(data)[-which(colnames(data) %in% c(formula.vars[1],locations))]
-  if (is.null(discmethod)) {discmethod = rep('quantile',length(xname))}
-  qv = vector("list",length = length(xname))
-  for (i in seq_along(xname)){
-    qv[[i]] = psmd_pseudop(
-      formula = paste(yname,'~',xname[i]),
-      data = dplyr::select(data,
-                           dplyr::all_of(c(yname,xname[i],locations))),
-      wt = wt, locations = locations, discnum = discnum, discmethod = discmethod[i],
-      cores = cores, seed = seed, permutations = permutations, ...)
+  xname = colnames(data)[-which(colnames(data) == yname)]
+  if (is.null(discvar)) {
+    xdiscname = xname
+    xundiscname = NULL
+  } else {
+    xundiscname = xname[-which(xname %in% discvar)]
+  }
+  if (length(discmethod) == 1) {discmethod = rep('quantile',length(xdiscname))}
+  qv_disc = vector("list",length = length(xdiscname))
+  for (i in seq_along(xdiscname)){
+    qv_disc[[i]] = psmd_pseudop(data[,yname,drop=TRUE],
+                                data[,xdiscname[i],drop=TRUE],
+                                wt_spade, discnum, discmethod[i],
+                                cores, seed, permutations, ...)
+  }
+  if (!is.null(xundiscname)) {
+    qv_undisc = vector("list",length = length(xundiscname))
+    for (i in seq_along(xundiscname)){
+      qv_undisc[[i]] = psd_pseudop(data[,yname,drop=TRUE],
+                                   data[,xundiscname[i],drop=TRUE],
+                                   wt_spade, cores, seed, permutations)
+    }
+    qv = purrr::list_cbind(c(qv_disc,qv_undisc))
+    xname = c(xdiscname,xundiscname)
+  } else {
+    qv = qv_disc
   }
   res = purrr::list_rbind(qv) %>%
     dplyr::mutate(variable = xname) %>%
